"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var HTMLElement_1 = __importDefault(require("../html-element/HTMLElement"));
var ValidityState_1 = __importDefault(require("./ValidityState"));
var DOMException_1 = __importDefault(require("../../exception/DOMException"));
var DOMExceptionNameEnum_1 = __importDefault(require("../../exception/DOMExceptionNameEnum"));
var Event_1 = __importDefault(require("../../event/Event"));
var HTMLInputElementValueSanitizer_1 = __importDefault(require("./HTMLInputElementValueSanitizer"));
var HTMLInputElementSelectionModeEnum_1 = __importDefault(require("./HTMLInputElementSelectionModeEnum"));
var HTMLInputElementSelectionDirectionEnum_1 = __importDefault(require("./HTMLInputElementSelectionDirectionEnum"));
/**
 * HTML Input Element.
 *
 * Reference:
 * https://developer.mozilla.org/en-US/docs/Web/API/HTMLInputElement
 *
 * Used as reference for some of the logic (like selection range):
 * https://github.com/jsdom/jsdom/blob/master/lib/jsdom/living/nodes/nodes/HTMLInputElement-impl.js (MIT licensed)
 */
var HTMLInputElement = /** @class */ (function (_super) {
    __extends(HTMLInputElement, _super);
    function HTMLInputElement() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        // Related to parent form.
        _this.formAction = '';
        _this.formMethod = '';
        _this.formNoValidate = false;
        // Any type of input
        _this.name = '';
        _this._type = 'text';
        _this.disabled = false;
        _this.autofocus = false;
        _this.required = false;
        _this._value = '';
        // Type specific: checkbox/radio
        _this.checked = false;
        _this.defaultChecked = false;
        _this.indeterminate = false;
        // Type specific: image
        _this.alt = '';
        _this.height = 0;
        _this.src = '';
        _this.width = 0;
        // Type specific: file
        _this.files = [];
        _this.accept = '';
        _this.allowdirs = '';
        // Type specific: text/number
        _this.autocomplete = '';
        _this.min = '';
        _this.max = '';
        _this.minLength = -1;
        _this.maxLength = -1;
        _this.pattern = '';
        _this.placeholder = '';
        _this.readOnly = false;
        _this.size = 0;
        // Type specific: text/password/search/tel/url/week/month
        _this._selectionStart = 0;
        _this._selectionEnd = 0;
        _this._selectionDirection = HTMLInputElementSelectionDirectionEnum_1.default.none;
        // Email
        _this.multiple = false;
        // Not categorized
        _this.defaultValue = '';
        _this.step = '';
        _this.inputmode = '';
        return _this;
    }
    Object.defineProperty(HTMLInputElement.prototype, "value", {
        /**
         * Returns value.
         *
         * @return Value.
         */
        get: function () {
            switch (this.type) {
                case 'hidden':
                case 'submit':
                case 'image':
                case 'reset':
                case 'button':
                    return this.getAttributeNS(null, 'value') || '';
                case 'checkbox':
                case 'radio':
                    var attritube = this.getAttributeNS(null, 'value');
                    return attritube !== null ? attritube : 'on';
                case 'file':
                    return this.files.length > 0 ? '/fake/path/' + this.files[0].name : '';
            }
            return this._value;
        },
        /**
         * Sets value.
         *
         * @param value Value.
         */
        set: function (value) {
            switch (this._type) {
                case 'hidden':
                case 'submit':
                case 'image':
                case 'reset':
                case 'button':
                case 'checkbox':
                case 'radio':
                    this.setAttributeNS(null, 'value', value);
                    break;
                case 'file':
                    if (value !== '') {
                        throw new DOMException_1.default('Input elements of type "file" may only programmatically set the value to empty string.', DOMExceptionNameEnum_1.default.invalidStateError);
                    }
                    break;
                default:
                    var oldValue = this._value;
                    this._value = HTMLInputElementValueSanitizer_1.default.sanitize(this, value);
                    if (oldValue !== this._value) {
                        this._selectionStart = value.length;
                        this._selectionEnd = value.length;
                        this._selectionDirection = HTMLInputElementSelectionDirectionEnum_1.default.none;
                    }
                    break;
            }
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(HTMLInputElement.prototype, "type", {
        /**
         * Returns type.
         *
         * @param type Type.
         */
        get: function () {
            return this._type;
        },
        /**
         * Sets type.
         *
         * @param type Type.
         */
        set: function (type) {
            this._type = type.toLowerCase();
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(HTMLInputElement.prototype, "selectionStart", {
        /**
         * Returns selection start.
         *
         * @return Selection start.
         */
        get: function () {
            if (!this._isSelectionSupported()) {
                return null;
            }
            return this._selectionStart;
        },
        /**
         * Sets selection start.
         *
         * @param start Start.
         */
        set: function (start) {
            if (!this._isSelectionSupported()) {
                throw new DOMException_1.default("The input element's type (" + this.type + ") does not support selection.", DOMExceptionNameEnum_1.default.invalidStateError);
            }
            this.setSelectionRange(start, Math.max(start, this._selectionEnd), this._selectionDirection);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(HTMLInputElement.prototype, "selectionEnd", {
        /**
         * Returns selection end.
         *
         * @return Selection end.
         */
        get: function () {
            if (!this._isSelectionSupported()) {
                return null;
            }
            return this._selectionEnd;
        },
        /**
         * Sets selection end.
         *
         * @param end End.
         */
        set: function (end) {
            if (!this._isSelectionSupported()) {
                throw new DOMException_1.default("The input element's type (" + this.type + ") does not support selection.", DOMExceptionNameEnum_1.default.invalidStateError);
            }
            this.setSelectionRange(this._selectionStart, end, this._selectionDirection);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(HTMLInputElement.prototype, "selectionDirection", {
        /**
         * Returns selection direction.
         *
         * @return Selection direction.
         */
        get: function () {
            if (!this._isSelectionSupported()) {
                return null;
            }
            return this._selectionDirection;
        },
        /**
         * Sets selection direction.
         *
         * @param direction Direction.
         */
        set: function (direction) {
            if (!this._isSelectionSupported()) {
                throw new DOMException_1.default("The input element's type (" + this.type + ") does not support selection.", DOMExceptionNameEnum_1.default.invalidStateError);
            }
            this.setSelectionRange(this._selectionStart, this._selectionEnd, direction);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(HTMLInputElement.prototype, "form", {
        /**
         * Returns the parent form element.
         *
         * @return Form.
         */
        get: function () {
            var parent = this.parentNode;
            while (parent && parent.tagName !== 'FORM') {
                parent = this.parentNode;
            }
            return parent;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(HTMLInputElement.prototype, "validity", {
        /**
         * Returns validity state.
         *
         * @returns Validity state.
         */
        get: function () {
            return new ValidityState_1.default(this);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(HTMLInputElement.prototype, "validationMessage", {
        /**
         * Returns validity message.
         *
         * @return Validation message.
         */
        get: function () {
            return null;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(HTMLInputElement.prototype, "willValidate", {
        /**
         * Returns "true" if it will validate.
         *
         * @return "true" if it will validate.
         */
        get: function () {
            return (this.type !== 'hidden' &&
                this.type !== 'reset' &&
                this.type !== 'button' &&
                !this.disabled &&
                !this['readOnly']);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(HTMLInputElement.prototype, "valueAsDate", {
        /**
         * Returns value as Date.
         *
         * @returns Date.
         */
        get: function () {
            return this.value ? new Date(this.value) : null;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(HTMLInputElement.prototype, "valueAsNumber", {
        /**
         * Returns value as number.
         *
         * @return Number.
         */
        get: function () {
            return this.value ? parseFloat(this.value) : NaN;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Set selection range.
     *
     * @param start Start.
     * @param end End.
     * @param [direction="none"] Direction.
     */
    HTMLInputElement.prototype.setSelectionRange = function (start, end, direction) {
        if (direction === void 0) { direction = 'none'; }
        if (!this._isSelectionSupported()) {
            throw new DOMException_1.default("The input element's type (" + this.type + ") does not support selection.", DOMExceptionNameEnum_1.default.invalidStateError);
        }
        this._selectionEnd = Math.min(end, this.value.length);
        this._selectionStart = Math.min(start, this._selectionEnd);
        this._selectionDirection =
            direction === HTMLInputElementSelectionDirectionEnum_1.default.forward ||
                direction === HTMLInputElementSelectionDirectionEnum_1.default.backward
                ? direction
                : HTMLInputElementSelectionDirectionEnum_1.default.none;
        this.dispatchEvent(new Event_1.default('select', { bubbles: true, cancelable: true }));
    };
    /**
     * Set range text.
     *
     * @param replacement Replacement.
     * @param [start] Start.
     * @param [end] End.
     * @param [direction] Direction.
     */
    HTMLInputElement.prototype.setRangeText = function (replacement, start, end, selectionMode) {
        if (start === void 0) { start = null; }
        if (end === void 0) { end = null; }
        if (selectionMode === void 0) { selectionMode = HTMLInputElementSelectionModeEnum_1.default.preserve; }
        if (!this._isSelectionSupported()) {
            throw new DOMException_1.default("The input element's type (" + this.type + ") does not support selection.", DOMExceptionNameEnum_1.default.invalidStateError);
        }
        if (start === null) {
            start = this._selectionStart;
        }
        if (end === null) {
            end = this._selectionEnd;
        }
        if (start > end) {
            throw new DOMException_1.default('The index is not in the allowed range.', DOMExceptionNameEnum_1.default.invalidStateError);
        }
        start = Math.min(start, this.value.length);
        end = Math.min(end, this.value.length);
        var val = this.value;
        var selectionStart = this._selectionStart;
        var selectionEnd = this._selectionEnd;
        this.value = val.slice(0, start) + replacement + val.slice(end);
        var newEnd = start + this.value.length;
        switch (selectionMode) {
            case HTMLInputElementSelectionModeEnum_1.default.select:
                this.setSelectionRange(start, newEnd);
                break;
            case HTMLInputElementSelectionModeEnum_1.default.start:
                this.setSelectionRange(start, start);
                break;
            case HTMLInputElementSelectionModeEnum_1.default.end:
                this.setSelectionRange(newEnd, newEnd);
                break;
            default:
                var delta = replacement.length - (end - start);
                if (selectionStart > end) {
                    selectionStart += delta;
                }
                else if (selectionStart > start) {
                    selectionStart = start;
                }
                if (selectionEnd > end) {
                    selectionEnd += delta;
                }
                else if (selectionEnd > start) {
                    selectionEnd = newEnd;
                }
                this.setSelectionRange(selectionStart, selectionEnd);
                break;
        }
    };
    /**
     * Checks validity.
     *
     * @return "true" if validation does'nt fail.
     */
    HTMLInputElement.prototype.checkValidity = function () {
        return true;
    };
    /**
     * Removes an Attr node.
     *
     * @override
     * @param attribute Attribute.
     */
    HTMLInputElement.prototype.removeAttributeNode = function (attribute) {
        _super.prototype.removeAttributeNode.call(this, attribute);
        switch (attribute.name) {
            case 'height': // number
            case 'width': // number
            case 'size': // number
                this[attribute.name] = 0;
                break;
            case 'selectionstart': // number
                this.selectionStart = 0;
                break;
            case 'selectionend': // number
                this.selectionEnd = 0;
                break;
            case 'minlength': // number
                this.minLength = -1;
                break;
            case 'maxlength': // number
                this.maxLength = -1;
                break;
            case 'type': // string
                this[attribute.name] = 'text';
                break;
            case 'name': // string
            case 'alt': // string
            case 'min': // string
            case 'max': // string
            case 'pattern': // string
            case 'placeholder': // string
            case 'step': // string
            case 'inputmode': // string
            case 'accept': // string
            case 'allowdirs': // string
            case 'autocomplete': // string
            case 'src': // string
                this[attribute.name] = '';
                break;
            case 'value': // string
                if (this._value === attribute.value && this._isPrivateValueSupported()) {
                    this.value = '';
                }
                break;
            case 'defaultvalue': // string
                this.defaultValue = '';
                break;
            case 'readonly': //  boolean
                this.readOnly = false;
            case 'disabled': // boolean
            case 'autofocus': // boolean
            case 'required': // boolean
            case 'indeterminate': // boolean
            case 'multiple': // boolean
            case 'checked': // boolean
                this[attribute.name] = false;
                break;
        }
    };
    /**
     * The setAttributeNode() method adds a new Attr node to the specified element.
     *
     * @override
     * @param attribute Attribute.
     * @returns Replaced attribute.
     */
    HTMLInputElement.prototype.setAttributeNode = function (attribute) {
        var replacedAttribute = _super.prototype.setAttributeNode.call(this, attribute);
        switch (attribute.name) {
            case 'height': // number
            case 'width': // number
            case 'size': // number
                this[attribute.name] = !!attribute.value ? Number(attribute.value) : 0;
                break;
            case 'minlength': // number
                this.minLength = !!attribute.value ? Number(attribute.value) : -1;
                break;
            case 'maxlength': // number
                this.maxLength = !!attribute.value ? Number(attribute.value) : -1;
                break;
            case 'type': // string
            case 'name': // string
            case 'alt': // string
            case 'min': // string
            case 'max': // string
            case 'pattern': // string
            case 'placeholder': // string
            case 'step': // string
            case 'inputmode': // string
            case 'accept': // string
            case 'allowdirs': // string
            case 'autocomplete': // string
            case 'src': // string
                this[attribute.name] = attribute.value || '';
                break;
            case 'value': // string
                if (!this._value && this._isPrivateValueSupported()) {
                    this.value = attribute.value || '';
                }
                break;
            case 'defaultvalue': // string
                this.defaultValue = attribute.value || '';
                break;
            case 'readonly': //  boolean
                this.readOnly = attribute.value !== null;
                break;
            case 'disabled': // boolean
            case 'autofocus': // boolean
            case 'required': // boolean
            case 'indeterminate': // boolean
            case 'multiple': // boolean
            case 'checked': // boolean
                this[attribute.name] = attribute.value !== null;
                break;
        }
        return replacedAttribute;
    };
    /**
     * Clones a node.
     *
     * @override
     * @param [deep=false] "true" to clone deep.
     * @return Cloned node.
     */
    HTMLInputElement.prototype.cloneNode = function (deep) {
        if (deep === void 0) { deep = false; }
        var clone = _super.prototype.cloneNode.call(this, deep);
        clone.formAction = this.formAction;
        clone.formMethod = this.formMethod;
        clone.formNoValidate = this.formNoValidate;
        clone.name = this.name;
        clone.type = this.type;
        clone.disabled = this.disabled;
        clone.autofocus = this.autofocus;
        clone.required = this.required;
        clone._value = this._value;
        clone.checked = this.checked;
        clone.defaultChecked = this.defaultChecked;
        clone.indeterminate = this.indeterminate;
        clone.alt = this.alt;
        clone.height = this.height;
        clone.src = this.src;
        clone.width = this.width;
        clone.accept = this.accept;
        clone.allowdirs = this.allowdirs;
        clone.autocomplete = this.autocomplete;
        clone.min = this.min;
        clone.max = this.max;
        clone.minLength = this.minLength;
        clone.maxLength = this.maxLength;
        clone.pattern = this.pattern;
        clone.placeholder = this.placeholder;
        clone.readOnly = this.readOnly;
        clone.size = this.size;
        clone.selectionStart = this.selectionStart;
        clone.selectionEnd = this.selectionEnd;
        clone.selectionDirection = this.selectionDirection;
        clone.defaultValue = this.defaultValue;
        clone.multiple = this.multiple;
        clone.step = this.step;
        clone.inputmode = this.inputmode;
        return clone;
    };
    /**
     * Checks if private value is supported.
     *
     * @returns "true" if private value is supported.
     */
    HTMLInputElement.prototype._isPrivateValueSupported = function () {
        return (this.type !== 'hidden' &&
            this.type !== 'submit' &&
            this.type !== 'image' &&
            this.type !== 'reset' &&
            this.type !== 'button' &&
            this.type !== 'checkbox' &&
            this.type !== 'radio' &&
            this.type !== 'file');
    };
    /**
     * Checks is selection is supported.
     *
     * @returns "true" if selection is supported.
     */
    HTMLInputElement.prototype._isSelectionSupported = function () {
        return (this.type === 'text' ||
            this.type === 'search' ||
            this.type === 'url' ||
            this.type === 'tel' ||
            this.type === 'password');
    };
    return HTMLInputElement;
}(HTMLElement_1.default));
exports.default = HTMLInputElement;
//# sourceMappingURL=HTMLInputElement.js.map